from contextlib import contextmanager
import os
from typing import Generator
from boilrpy.config import Config


class FileWriterError(Exception):
    """Custom exception for FileWriter errors"""


class FileWriter:
    """Class to write files and directories."""

    def __init__(self, charset: str = Config().get_charset()) -> None:
        self.charset = charset

    @contextmanager
    def open_file(self, filename: str, mode: str = "w") -> Generator:
        """Context manager to open and close a file.

        Args:
            filename (str): The name of the file to open.
            mode (str, optional): The mode to open the file in. Defaults to "w".

        Yields:
            file: The opened file object.
        """
        file = None
        try:
            file = open(filename, mode, encoding=self.charset)
            yield file
        except IOError as e:
            raise FileWriterError(f"Error opening file {filename}: {e}") from e
        finally:
            if file:
                file.close()

    def write_file(self, filename: str, content: str) -> None:
        """Write content to a file.

        Args:
            filename (str): The name of the file to write to.
            content (str): The content to write to the file.
        """
        try:
            with self.open_file(filename, "w") as file:
                file.write(content)
        except FileWriterError as e:
            raise FileWriterError(f"Error writing to file {filename}: {e}") from e

    def create_directory(self, directory: str, exist_ok: bool = True) -> None:
        """Create a directory.

        Args:
            directory (str): The name of the directory to create.
            exist_ok (bool, optional): Whether to raise an exception
            if the directory already exists. Defaults to True.
        """
        try:
            os.makedirs(directory, exist_ok=exist_ok)
        except OSError as e:
            raise FileWriterError(f"Error creating directory {directory}: {e}") from e



from boilrpy.file_generators.base_generator import BaseGenerator


class ReadmeGenerator(BaseGenerator):
    """
    Generator for README.md file.
    """

    DEP_MANAGER_CONFIG = {
        "poetry": {
            "setup": (
                "### Using Poetry\n\n"
                "Install dependencies:\n\n"
                "```bash\n"
                "poetry install\n"
                "```\n\n"
                "Activate the virtual environment:\n\n"
                "```bash\n"
                "poetry env activate\n"
                "```"
            ),
            "run_command": "poetry run python main.py",
            "flask_command": "poetry run flask run",
            "test_command": "poetry run pytest",
        },
        "pip": {
            "setup": (
                "### Using pip and venv\n\n"
                "Create a virtual environment:\n\n"
                "On Linux and macOS:\n\n"
                "```bash\n"
                "python3 -m venv venv\n"
                "```\n\n"
                "On Windows:\n\n"
                "```bash\n"
                "python -m venv venv\n"
                "```\n\n"
                "Activate the virtual environment:\n\n"
                "On Linux and macOS:\n\n"
                "```bash\n"
                "source venv/bin/activate\n"
                "```\n\n"
                "On Windows:\n\n"
                "```bash\n"
                "venv\\Scripts\\activate\n"
                "```\n\n"
                "Install dependencies:\n\n"
                "```bash\n"
                "pip install -r requirements.txt\n"
                "```"
            ),
            "run_command": "python main.py",
            "flask_command": "flask run",
            "test_command": "pytest",
        },
        "uv": {
            "setup": (
                "### Using uv\n\n"
                "Create a virtual environment and install dependencies:\n\n"
                "```bash\n"
                "uv venv\n"
                "```\n\n"
                "Activate the virtual environment:\n\n"
                "On Linux and macOS:\n\n"
                "```bash\n"
                "source .venv/bin/activate\n"
                "```\n\n"
                "On Windows:\n\n"
                "```bash\n"
                ".venv\\Scripts\\activate\n"
                "```\n\n"
                "Install dependencies:\n\n"
                "```bash\n"
                "uv pip install -r requirements.txt\n"
                "```"
            ),
            "run_command": "uv run python main.py",
            "flask_command": "uv run flask run",
            "test_command": "uv run pytest",
        },
        "conda": {
            "setup": (
                "### Using Conda\n\n"
                "Create a conda environment:\n\n"
                "```bash\n"
                "conda create -n myproject python=3.11\n"
                "```\n\n"
                "Activate the environment:\n\n"
                "```bash\n"
                "conda activate myproject\n"
                "```\n\n"
                "Install dependencies:\n\n"
                "```bash\n"
                "conda install --file requirements.txt\n"
                "```"
            ),
            "run_command": "python main.py",
            "flask_command": "flask run",
            "test_command": "pytest",
        },
    }

    def generate(self, *args, **kwargs) -> str:
        """
        Generate the content for README.md file.

        :param project_info: Dictionary containing project information
        :param kwargs: Additional keyword arguments
        :return: Content of README.md file
        """
        project_info = args[0] if args else {}

        dep_manager = project_info.get("dependencies_manager", "pip").lower()
        manager_config = self.DEP_MANAGER_CONFIG.get(
            dep_manager,
            self.DEP_MANAGER_CONFIG["pip"],  # Fallback on pip if unknown
        )

        setup_instructions = manager_config["setup"]

        if project_info.get("use_flask"):
            usage_instructions = manager_config["flask_command"]
        else:
            usage_instructions = manager_config["run_command"]

        testing_instructions = ""
        if project_info.get("create_tests"):
            testing_instructions = (
                f"\n## Testing\n\n```bash\n{manager_config['test_command']}\n```"
            )

        pylint_instructions = ""
        if project_info.get("use_pylint"):
            pylint_instructions = "\n## Pylint\n\n```bash\npylint .\n```"

        license_info = ""
        if project_info.get("license") and project_info["license"] != "None":
            license_info = (
                f"\n## License\n\nThis project is licensed under the "
                f"{project_info['license']} License."
            )

        template = """
# ${name}

${description}

## Setup

${setup_instructions}

## Usage

To run the project, use the following comand:

${usage_instructions}

${testing_instructions}

${pylint_instructions}

## Contributing

1. Fork the repository
2. Create a new branch (`git checkout -b feature/your-feature`)
3. Make your changes
4. Commit your changes (`git commit -am 'Add some feature'`)
5. Push to the branch (`git push origin feature/your-feature`)
6. Create a new Pull Request

${license_info}
"""
        return self.render_template(
            template,
            name=project_info["name"].title(),
            description=project_info["description"],
            setup_instructions=setup_instructions,
            usage_instructions=usage_instructions,
            testing_instructions=testing_instructions,
            pylint_instructions=pylint_instructions,
            license_info=license_info,
        ).lstrip()



import subprocess
from boilrpy.dependency_creators.base_dependency_creator import (
    BaseDependencyCreator,
    DependencyCreatorNotFoundError,
    DependencyCreatorError,
)


class CondaCreator(BaseDependencyCreator):
    """Class to create a new conda project."""

    def create_dependency_file(self, project_info: dict) -> None:
        """Create a new conda project with environment.yml.

        Args:
            project_info (dict): Dictionary containing project information
        """
        try:
            packages, dev_packages = self._create_packages(project_info)

            # Create environment.yml
            self._create_environment_file(project_info, packages, dev_packages)

            print("\nTo create and activate the conda environment:")
            print("1. conda env create -f environment.yml")
            print(f"2. conda activate {project_info['name']}")

        except Exception as e:
            raise DependencyCreatorError(f"conda initialization failed: {e}") from e

    def install_dependencies(self, packages: list, dev_packages: list) -> None:
        """Install dependencies using conda.

        Args:
            packages (list): List of regular packages
            dev_packages (list): List of development packages
        """
        try:
            all_packages = packages + dev_packages
            if all_packages:
                subprocess.run(["conda", "install", "-y"] + all_packages, check=True)
        except subprocess.CalledProcessError as e:
            raise DependencyCreatorError(f"Failed to install dependencies: {e}") from e
        except FileNotFoundError as exc:
            raise DependencyCreatorNotFoundError(
                "conda not found. Please install Anaconda or Miniconda."
            ) from exc

    def _create_environment_file(
        self, project_info: dict, packages: list, dev_packages: list
    ) -> None:
        """Create environment.yml file.

        Args:
            project_info (dict): Dictionary containing project information
            packages (list): List of regular packages
            dev_packages (list): List of development packages
        """
        all_packages = packages + dev_packages

        environment_content = f"""name: {project_info["name"]}
channels:
  - conda-forge
  - defaults
dependencies:
  - python={project_info.get("python_version", "3.11")}
"""

        if all_packages:
            for package in all_packages:
                environment_content += f"  - {package}\n"

        environment_content += """
  - pip
  - pip:
    # Add pip-only packages here
"""

        with open("environment.yml", "w", encoding=self.charset) as f:
            f.write(environment_content)



import re

NAME_PATTERN = r"^[a-zA-Z0-9_\-\s]+$"
VERSION_PATTERN = (
    r"^\s*(0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)" + r"(?:-(alpha|beta|rc))?\s*$"
)


class InputValidator:
    """
    Class for validating user input.
    """

    DEFAULT_VERSION = "0.1.0"

    @staticmethod
    def validate_project_name(name: str) -> bool:
        """
        Validate project name.

        Args:
            name (str): Project name.

        Returns:
            bool: Whether the project name is valid.
        """
        return re.match(NAME_PATTERN, name) is not None

    @staticmethod
    def validate_version(version: str = DEFAULT_VERSION) -> bool:
        """
        Validate project version.

        Args:
            version (str): Project version.

        Returns:
            bool: Whether the project version is valid.
        """
        return True if version == "" else re.match(VERSION_PATTERN, version) is not None



"""
Dependency Creators Module

This module provides creators for different Python dependency managers.
Each creator implements the BaseDependencyCreator interface and handles
the specifics of setting up projects with that dependency manager.

Available Creators:
    - PoetryCreator: For Poetry (professional projects)
    - PipCreator: For pip (simple projects)
    - UvCreator: For uv (fast modern projects)
    - CondaCreator: For Conda (data science projects)

Example:
    >>> from boilrpy.dependency_creators import DependencyCreatorFactory
    >>> from boilrpy.config import Config
    >>>
    >>> config = Config()
    >>> creator = DependencyCreatorFactory.create("poetry", config)
    >>> creator.create_dependency_file(project_info)
"""

# Base classes and exceptions
from boilrpy.dependency_creators.base_dependency_creator import (
    BaseDependencyCreator,
    DependencyCreatorNotFoundError,
    DependencyCreatorError,
)

# Concrete creators
from boilrpy.dependency_creators.poetry_creator import PoetryCreator
from boilrpy.dependency_creators.pip_creator import PipCreator
from boilrpy.dependency_creators.uv_creator import UvCreator
from boilrpy.dependency_creators.conda_creator import CondaCreator

# Factory
from boilrpy.dependency_creators.factory import DependencyCreatorFactory

# Utilities
from boilrpy.dependency_creators.checker import DependencyManagerChecker

# Public API
__all__ = [
    # Base
    "BaseDependencyCreator",
    "DependencyCreatorNotFoundError",
    "DependencyCreatorError",
    # Creators
    "PoetryCreator",
    "PipCreator",
    "UvCreator",
    "CondaCreator",
    # Factory
    "DependencyCreatorFactory",
    # Utilities
    "DependencyManagerChecker",
]



from functools import wraps
from colorama import init, Fore, Style

init(autoreset=True)


class ColorDecorator:
    """Decorator to color input and output messages."""

    @staticmethod
    def color_input(color=Fore.GREEN):
        """Decorator to color input messages."""

        def decorator(func):
            @wraps(func)
            def wrapper(*args, **kwargs):
                prompt = args[0] if args else kwargs.get("prompt", "")
                colored_prompt = f"{color}{prompt}{Style.RESET_ALL}"
                return func(colored_prompt, *args[1:], **kwargs)

            return wrapper

        return decorator

    @staticmethod
    def color_output(color=Fore.GREEN):
        """Decorator to color output messages."""

        def decorator(func):
            @wraps(func)
            def wrapper(*args, **kwargs):
                message = args[0] if args else kwargs.get("message", "")
                colored_message = f"{color}{message}{Style.RESET_ALL}"
                return func(colored_message, *args[1:], **kwargs)

            return wrapper

        return decorator



from boilrpy.file_generators.base_generator import BaseGenerator


class FlaskGenerator(BaseGenerator):
    """
    Flask structure generator.
    """

    def generate(self, *args, **kwargs) -> None:
        project_info = args[0] if args else {}
        self.generate_app_file()
        self.generate_base_template(project_info)
        self.generate_index_template(project_info)
        self.generate_dot_env_file()
        self.generate_style_file()
        self.generate_script_file()

    def generate_app_file(self) -> str:
        """
        Generate app file content.

        :return: Content of app file
        """
        return """from flask import Flask, render_template
from dotenv import load_dotenv


load_dotenv()
app = Flask(__name__)


@app.route('/')
def index():
    return render_template('index.html')


if __name__ == '__main__':
    app.run(host="0.0.0.0", port=5000, debug=True)
"""

    def generate_base_template(self, project_info: dict) -> str:
        """
        Generate base template content.

        :args project_info: Dictionary containing project information

        :return: Content of base template
        """
        template = """<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>{% block title %}${project_name}{% endblock %}</title>
    <link rel="stylesheet" href="{{ url_for('static', filename='css/style.css') }}">
</head>
<body>
    <header>
        <!-- Add your header content here -->
    </header>
    
    <main>
        {% block content %}{% endblock %}
    </main>
    
    <footer>
        <!-- Add your footer content here -->
    </footer>
    
    <script src="{{ url_for('static', filename='js/script.js') }}"></script>
</body>
</html>
"""
        return self.render_template(template, project_name=project_info["name"].title())

    def generate_index_template(self, project_info: dict) -> str:
        """
        Generate index template content.

        :args project_info: Dictionary containing project information

        :return: Content of index template
        """
        template = """{% extends "base.html" %}

{% block content %}
<h1>Welcome to ${project_name}</h1>
<p>${description}</p>
{% endblock %}
"""
        return self.render_template(
            template,
            project_name=project_info["name"].capitalize(),
            description=project_info["description"].capitalize(),
        )

    def generate_dot_env_file(self) -> str:
        """
        Generate .env file content.

        :return: Content of .env file
        """
        return """FLASK_APP=app.py
FLASK_ENV=development
FLASK_DEBUG=1
"""

    def generate_style_file(self) -> str:
        """
        Generate style file content.

        :return: Content of style file
        """
        return """/* Add your custom styles here */
body {
    font-family: Arial, sans-serif;
    margin: 0;
    padding: 0;
    display: flex;
    justify-content: center;
    align-items: center;
    min-height: 100vh;
    background: linear-gradient(135deg, #00d2ff 0%, #3a47d5 100%);
    color: white;
}
.container {
    text-align: center;
    padding: 2rem;
    background-color: rgba(255, 255, 255, 0.1);
    border-radius: 10px;
    backdrop-filter: blur(10px);
    box-shadow: 0 4px 6px rgba(0, 0, 0, 0.1);
}
h1 {
    font-size: 2.5rem;
    margin-bottom: 1rem;
}
p {
    font-size: 1.2rem;
    line-height: 1.6;
}
"""

    def generate_script_file(self) -> str:
        """
        Generate script file content.

        :return: Content of script file
        """
        return """// Add your custom JavaScript here
console.log('Script loaded');
"""



import argparse
from boilrpy.cli import CLI
from boilrpy.config import Config
from boilrpy.project_creator import ProjectCreator
from boilrpy.dependency_creators.checker import DependencyManagerChecker


def run_cli(args):
    """Run the CLI with given arguments."""
    if args.check_deps:
        DependencyManagerChecker.print_status()
        return

    config = Config()
    cli = CLI(config)
    project_info = cli.gather_project_info()
    creator = ProjectCreator(config)
    creator.create_project(project_info)


def main():
    """CLI entry point."""
    parser = argparse.ArgumentParser(
        description="Boilrpy - Python Project Boilerplate Generator"
    )
    parser.add_argument(
        "--check-deps",
        action="store_true",
        help="Check which dependency managers are installed",
    )
    args = parser.parse_args()
    run_cli(args)


if __name__ == "__main__":  # pragma: no cover
    main()



from boilrpy.file_generators.base_generator import BaseGenerator


class PylintGenerator(BaseGenerator):
    """
    Generator for .pylintrc rules
    """

    def generate(self, *args, **kwargs) -> str:
        """
        Generate the content for .pylintrc file.

        :return: Content of .pylintrc file
        """
        return """[MASTER]
disable=missing-module-docstring, too-few-public-methods 
ignore=tests"""



from abc import ABC, abstractmethod
from typing import Any, Dict, Type
from boilrpy.file_generators.changelog_generator import ChangelogGenerator
from boilrpy.file_generators.dockerfile_generator import DockerfileGenerator
from boilrpy.file_generators.gitignore_generator import GitignoreGenerator
from boilrpy.file_generators.license_generator import LicenseGenerator
from boilrpy.file_generators.main_file_generator import MainFileGenerator
from boilrpy.file_generators.readme_generator import ReadmeGenerator
from boilrpy.file_generators.pylint_generator import PylintGenerator
from boilrpy.file_generators.flask_generator import FlaskGenerator
from boilrpy.file_generators.requirements_generator import RequirementsGenerator
from boilrpy.config import Config


class Generator(ABC):
    """
    Abstract class for generating project files.
    """

    @abstractmethod
    def generate(self, *args, **kwargs) -> str:
        """Generate the content of the file."""


class GeneratorFactory:
    """
    Factory class for creating generators.
    """

    _generators: Dict[str, Type[Generator]] = {
        "readme": ReadmeGenerator,
        "license": LicenseGenerator,
        "gitignore": GitignoreGenerator,
        "changelog": ChangelogGenerator,
        "main_file": MainFileGenerator,
        "dockerfile": DockerfileGenerator,
        "pylint": PylintGenerator,
        "flask": FlaskGenerator,
        "requirements": RequirementsGenerator,
    }

    @classmethod
    def create_generator(cls, generator_type: str, config: Dict[str, Any]) -> Generator:
        """
        Create a generator instance.

        Args:
            generator_type (str): The type of generator to create.
            config (Dict[str, Any]): The configuration dictionary.

        Returns:
            Generator: The created generator instance.

        Raises:
            ValueError: If the generator type is unknown.
        """
        generator_class = cls._generators.get(generator_type)
        if generator_class is None:
            raise ValueError(f"Unknown generator type: {generator_type}")
        return generator_class(config)


class FileGenerator:
    """
    Main class for generating project files.
    """

    def __init__(self, config: Config):
        self.config = config
        self.generators: Dict[str, Generator] = {}

    def _get_generator(self, generator_type: str) -> Generator:
        if generator_type not in self.generators:
            self.generators[generator_type] = GeneratorFactory.create_generator(
                generator_type, self.config
            )
        return self.generators[generator_type]

    def generate_readme(self, project_info: dict) -> str:
        """
        Generate README.md content.

        :param project_info: Dictionary containing project information
        :return: Content of README.md file
        """
        return self._get_generator("readme").generate(project_info)

    def generate_license(self, license_name: str, author: str) -> str:
        """
        Generate license file content.

        :param license_name: Name of the license
        :param author: Name of the author
        :return: Content of the license file
        """
        return self._get_generator("license").generate(license_name, author)

    def generate_gitignore(self) -> str:
        """
        Generate .gitignore file content.

        :return: Content of .gitignore file
        """
        return self._get_generator("gitignore").generate()

    def generate_changelog(self, version: str) -> str:
        """
        Generate CHANGELOG.md content.

        :param version: Version number
        :return: Content of CHANGELOG.md file
        """
        return self._get_generator("changelog").generate(version)

    def generate_main_file(self) -> str:
        """
        Generate main.py content.

        :return: Content of main.py file
        """
        return self._get_generator("main_file").generate()

    def generate_dockerfile(self, project_name: str, use_flask: bool) -> str:
        """
        Generate Dockerfile content.

        :return: Content of Dockerfile
        """
        return self._get_generator("dockerfile").generate_dockerfile(
            project_name, use_flask
        )

    def generate_dockerignore(self) -> str:
        """
        Generate .dockerignore content.

        :return: Content of .dockerignore
        """
        return self._get_generator("dockerfile").generate_dockerignore()

    def generate_pylint(self) -> str:
        """
        Generate .pylintrc content.

        :return: Content of .pylintrc
        """
        return self._get_generator("pylint").generate()

    def generate_requirements_txt(self, project_info: dict) -> str:
        """
        Generate requirements.txt content.

        :return: Content of requirements.txt
        """
        return self._get_generator("requirements").generate(project_info)

    def generate_flask_app_file(self) -> str:
        """
        Generate flask app file content.

        :return: Content of flask app file
        """
        return self._get_generator("flask").generate_app_file()

    def generate_base_template(self, project_info: dict) -> str:
        """
        Generate base template content.

        :return: Content of base template
        """
        return self._get_generator("flask").generate_base_template(project_info)

    def generate_index_template(self, project_info: dict) -> str:
        """
        Generate index template content.

        :return: Content of index template
        """
        return self._get_generator("flask").generate_index_template(project_info)

    def generate_dot_env_file(self) -> str:
        """
        Generate .env file content.

        :return: Content of .env file
        """
        return self._get_generator("flask").generate_dot_env_file()

    def generate_style_file(self) -> str:
        """
        Generate style file content.

        :return: Content of style file
        """
        return self._get_generator("flask").generate_style_file()

    def generate_script_file(self) -> str:
        """
        Generate script file content.

        :return: Content of script file
        """
        return self._get_generator("flask").generate_script_file()



class Config:
    """
    BoilrPy configuration class.
    """

    def __init__(self):
        self.available_licenses = ["MIT", "Apache", "GPL", "BSD", "None"]
        self.available_dep_managers = ["pip", "poetry", "uv", "conda"]
        self.use_camel_case = False  # 'camelCase' or 'snake_case'
        self.charset = "utf-8"
        self.default_version = "0.1.0"
        self.python_version = "3.11"

    def get_available_licenses(self) -> list:
        """Returns a list of available licenses."""
        return self.available_licenses

    def get_available_dep_managers(self) -> list:
        """Returns a list of available dependency managers."""
        return self.available_dep_managers

    def get_use_camel_case(self) -> bool:
        """Returns whether to use camelCase or snake_case."""
        return self.use_camel_case

    def get_charset(self) -> str:
        """Returns the charset."""
        return self.charset

    def get_default_version(self) -> str:
        """Returns the default version."""
        return self.default_version



import re


class StringFormatter:
    """
    A utility class for formatting strings.
    """

    @staticmethod
    def to_camel_case(string: str) -> str:
        """
        Convert a string to camelCase.

        Args:
            string (str): The input string.

        Returns:
            str: The string in camelCase format.
        """
        if re.match(r"^[a-z][a-zA-Z0-9]*$", string):
            return string
        if re.match(r"^[A-Z][a-zA-Z0-9]*$", string):
            return string[0].lower() + string[1:]
        words = re.findall(r"[A-Za-z0-9]+", string)
        return words[0].lower() + "".join(word.capitalize() for word in words[1:])

    @staticmethod
    def to_snake_case(string: str) -> str:
        """
        Convert a string to snake_case.

        Args:
            string (str): The input string.

        Returns:
            str: The string in snake_case format.
        """
        string = string.strip()
        if re.match(r"^[a-z0-9_-]+$", string):
            return string
        s1 = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", string)
        s2 = re.sub("([a-z0-9])([A-Z])", r"\1_\2", s1)
        return re.sub("[^a-z0-9]+", "_", s2.lower())

    @staticmethod
    def format_project_name(project_name: str, use_camel_case: bool) -> str:
        """
        Format a project name to either camelCase or snake_case.

        Args:
            project_name (str): The input project name.
            use_camel_case (bool): Whether to use camelCase or snake_case.

        Returns:
            str: The formatted project name.
        """
        if use_camel_case:
            return StringFormatter.to_camel_case(project_name)
        return StringFormatter.to_snake_case(project_name)



"""Utility to check if dependency managers are installed."""

import subprocess
from typing import Dict, Optional


class DependencyManagerChecker:
    """Check availability of dependency managers on the system."""

    @staticmethod
    def check_manager(manager_name: str) -> tuple[bool, Optional[str]]:
        """Check if a dependency manager is installed.

        Args:
            manager_name: Name of the dependency manager to check

        Returns:
            tuple: (is_installed, version_string or None)
        """
        version_commands = {
            "pip": ["pip", "--version"],
            "poetry": ["poetry", "--version"],
            "uv": ["uv", "--version"],
            "conda": ["conda", "--version"],
        }

        command = version_commands.get(manager_name)
        if not command:
            return False, None

        try:
            result = subprocess.run(
                command, capture_output=True, text=True, check=True, timeout=5
            )
            return True, result.stdout.strip()
        except (
            FileNotFoundError,
            subprocess.CalledProcessError,
            subprocess.TimeoutExpired,
        ):
            return False, None

    @staticmethod
    def check_all_managers() -> Dict[str, tuple[bool, Optional[str]]]:
        """Check all supported dependency managers.

        Returns:
            dict: Dictionary mapping manager names to (is_installed, version)
        """
        managers = ["pip", "poetry", "uv", "conda"]
        return {
            manager: DependencyManagerChecker.check_manager(manager)
            for manager in managers
        }

    @staticmethod
    def get_available_managers() -> list[str]:
        """Get list of available (installed) dependency managers.

        Returns:
            list: List of installed manager names
        """
        all_managers = DependencyManagerChecker.check_all_managers()
        return [
            manager for manager, (installed, _) in all_managers.items() if installed
        ]

    @staticmethod
    def print_status() -> None:
        """Print status of all dependency managers."""
        print("\n" + "=" * 60)
        print("Dependency Managers Status")
        print("=" * 60)

        all_managers = DependencyManagerChecker.check_all_managers()

        for manager, (installed, version) in all_managers.items():
            status = "Installed" if installed else "Not installed"
            version_info = f" ({version})" if version else ""
            print(f"{manager:10} : {status}{version_info}")

        print("=" * 60)

        available = DependencyManagerChecker.get_available_managers()
        if available:
            print(f"\nAvailable managers: {', '.join(available)}")
        else:
            print("\n No dependency managers found!")
            print("Please install at least one dependency manager.")


if __name__ == "__main__":  # pragma: no cover
    # Run status check
    DependencyManagerChecker.print_status()



from abc import ABC, abstractmethod
from string import Template
from boilrpy.config import Config


class BaseGenerator(ABC):
    """
    Abstract base class for file generators.
    """

    def __init__(self, config: Config):
        self.config = config

    @abstractmethod
    def generate(self, *args, **kwargs) -> str:
        """
        Generate the file content.
        """

    def render_template(self, template: str, **kwargs) -> str:
        """
        Render a template with the given kwargs.
        """
        return Template(template).safe_substitute(**kwargs)



import sys
from colorama import Fore, Style
from boilrpy.input_validator import InputValidator
from boilrpy.decorators.color_decorator import ColorDecorator


class CLI:
    """
    CLI class to gather input from the user.
    """

    def __init__(self, config):
        self.config = config

    def gather_project_info(self) -> dict:
        """
        Gather project information from the user.

        Returns:
            dict: Dictionary containing project information.
        """
        print(
            f"\n{Fore.CYAN}{Style.BRIGHT}Welcome to boilrpy Project Creator!{Style.RESET_ALL}"
        )
        print(
            f"{Fore.YELLOW}Please provide the following information for your new project:"
            f"{Style.RESET_ALL}\n"
        )
        project_info = {}
        try:
            project_info["name"] = self._get_valid_input(
                "Enter project name: ", InputValidator.validate_project_name
            )

            project_info["description"] = input(
                f"{Fore.GREEN}Project description: {Style.RESET_ALL}"
            )
            project_info["version"] = self._get_valid_input(
                "Enter project version [0.1.0]: ", InputValidator.validate_version
            )
            project_info["author"] = (
                input(f"{Fore.GREEN}Author name: {Style.RESET_ALL}") or ""
            )
            project_info["license"] = self._choose_license()

            project_info["dependencies_manager"] = self._choose_dependencies_manager()

            project_info["use_docker"] = self._yes_no_question(
                "Generate Dockerfile? (y/n) [y]: ", "y"
            )
            project_info["create_tests"] = self._yes_no_question(
                "Create a test folder? (y/n) [y]: ", "y"
            )
            project_info["use_pylint"] = self._yes_no_question(
                "Use pylint in project? (y/n) [n]: ", "n"
            )
            project_info["use_flask"] = self._yes_no_question(
                "Use flask in project? (y/n) [n]: ", "n"
            )

            self._display_summary(project_info)
            return project_info
        except KeyboardInterrupt:
            print(f"\n{Fore.MAGENTA}Operation cancelled by user.{Style.RESET_ALL}")
            sys.exit(0)

    def _get_valid_input(self, prompt: str, validator: callable) -> str:
        while True:
            value = input(prompt)
            if validator(value):
                return value
            self.display_error("Invalid input. Please try again.")

    def _choose_license(self) -> str:
        licenses = self.config.get_available_licenses()
        print(f"\n{Fore.YELLOW}Available licenses:{Style.RESET_ALL}")
        for i, lic in enumerate(licenses, 1):
            print(f"{Fore.CYAN}{i}. {lic}{Style.RESET_ALL}")
        while True:
            choice = input("Choose a license (enter the number): ")
            try:
                index = int(choice) - 1
                if 0 <= index < len(licenses):
                    return licenses[index]
            except ValueError:
                pass
            self.display_error("Invalid choice. Please try again.")

    def _choose_dependencies_manager(self) -> str:
        dep_manager = self.config.get_available_dep_managers()
        print(f"\n{Fore.YELLOW}Available dependency managers:{Style.RESET_ALL}")
        for i, dep in enumerate(dep_manager, 1):
            print(f"{Fore.CYAN}{i}. {dep}{Style.RESET_ALL}")
        while True:
            choice = input("Choose a dependencies manager (enter the number): ")
            try:
                index = int(choice) - 1
                if 0 <= index < len(dep_manager):
                    return dep_manager[index]
            except ValueError:
                pass
            self.display_error("Invalid choice. Please try again.")

    def _yes_no_question(self, question: str, default_response: str = None) -> bool:
        default_hint = f" [{default_response}]" if default_response else ""

        while True:
            answer = input(f"{question} (y/n){default_hint}: ").strip().lower()
            if not answer and default_response:
                answer = default_response.lower()
            if answer in ["y", "yes"]:
                return True
            if answer in ["n", "no"]:
                return False
            self.display_error("Invalid input. Please enter 'y' or 'n'.")

    @ColorDecorator.color_output(Fore.BLUE)
    def _display_summary(self, project_info: dict):
        print(f"\n{Fore.CYAN}{Style.BRIGHT}Project Summary:{Style.RESET_ALL}")
        for key, value in project_info.items():
            print(f"{Fore.YELLOW}{key.capitalize()}: {Fore.WHITE}{value}")
        print("\n")

    @ColorDecorator.color_output(Fore.RED)
    def display_error(self, message: str):
        """Display an error message."""
        print(f"{Fore.RED}✘ Error: {message}{Style.RESET_ALL}")



import subprocess
from boilrpy.dependency_creators.base_dependency_creator import (
    BaseDependencyCreator,
    DependencyCreatorNotFoundError,
    DependencyCreatorError,
)


class UvCreator(BaseDependencyCreator):
    """Class to create a new uv project."""

    def create_dependency_file(self, project_info: dict) -> None:
        """Create a new uv project with requirements.txt.

        Args:
            project_info (dict): Dictionary containing project information
        """
        try:
            # Check if uv is installed first
            subprocess.run(["uv", "--version"], check=True, capture_output=True)
        except FileNotFoundError as exc:
            raise DependencyCreatorNotFoundError(
                "uv not found. Please install uv and try again.\n"
                "Installation options:\n"
                "  - Using pip: pip install uv\n"
                "  - Using pipx: pipx install uv\n"
                "  - Using cargo: cargo install --git https://github.com/astral-sh/uv uv\n"
                "  - Download from: https://github.com/astral-sh/uv/releases"
            ) from exc
        except subprocess.CalledProcessError as exc:
            raise DependencyCreatorNotFoundError(
                "uv not found or not working properly. Please install uv and try again.\n"
                "Installation options:\n"
                "  - Using pip: pip install uv\n"
                "  - Using pipx: pipx install uv\n"
                "  - Using cargo: cargo install --git https://github.com/astral-sh/uv uv\n"
                "  - Download from: https://github.com/astral-sh/uv/releases"
            ) from exc

        try:
            packages, dev_packages = self._create_packages(project_info)

            # Create virtual environment with uv
            subprocess.run(["uv", "venv"], check=True)

            # Create requirements.txt using base class method
            self._write_requirements_files(
                packages, dev_packages, create_empty_if_no_packages=False
            )

            # Install dependencies
            self.install_dependencies(packages, dev_packages)

        except subprocess.CalledProcessError as e:
            raise DependencyCreatorError(f"uv initialization failed: {e}") from e

    def install_dependencies(self, packages: list, dev_packages: list) -> None:
        """Install dependencies using uv.

        Args:
            packages (list): List of regular packages
            dev_packages (list): List of development packages
        """
        try:
            all_packages = packages + dev_packages
            if all_packages:
                subprocess.run(["uv", "pip", "install"] + all_packages, check=True)
        except subprocess.CalledProcessError as e:
            raise DependencyCreatorError(f"Failed to install dependencies: {e}") from e



from boilrpy.file_generators.base_generator import BaseGenerator


class GitignoreGenerator(BaseGenerator):
    """
    Generator for .gitignore file.
    """

    def generate(self, *args, **kwargs) -> str:
        """
        Generate the content for .gitignore file.

        :return: Content of .gitignore file
        """
        return """# Python
__pycache__/
*.py[cod]
*$py.class

# Virtual environments
venv/
.env
.venv
env/

# Poetry
.poetry/

# PyCharm
.idea/

# VS Code
.vscode/

# Jupyter Notebook
.ipynb_checkpoints

# Distribution / packaging
.Python
build/
develop-eggs/
dist/
downloads/
eggs/
.eggs/
lib/
lib64/
parts/
sdist/
var/
wheels/
*.egg-info/
.installed.cfg
*.egg

# Pytest
.pytest_cache/

# Coverage reports
htmlcov/
.coverage
.coverage.*
.cache

# Matplotlib
*.png

# macOS
.DS_Store

# Windows
Thumbs.db

# Output folder
output/
"""



import subprocess
from boilrpy.dependency_creators.base_dependency_creator import (
    BaseDependencyCreator,
    DependencyCreatorNotFoundError,
    DependencyCreatorError,
)


class PipCreator(BaseDependencyCreator):
    """Class to create a new pip project."""

    def create_dependency_file(self, project_info: dict) -> None:
        """Create a new pip project with requirements.txt.

        Args:
            project_info (dict): Dictionary containing project information
        """
        try:
            packages, dev_packages = self._create_packages(project_info)

            # Create requirements.txt files using base class method
            self._write_requirements_files(packages, dev_packages)

            # Note: We don't auto-install with pip to let user create venv first
            print("\nTo install dependencies:")
            print("1. Create a virtual environment:")
            print("   python -m venv venv")
            print("2. Activate it:")
            print("   source venv/bin/activate  # Linux/macOS")
            print("   venv\\Scripts\\activate     # Windows")
            print("3. Install dependencies:")
            print("   pip install -r requirements.txt")
            if dev_packages:
                print("   pip install -r requirements-dev.txt")

        except Exception as e:
            raise DependencyCreatorError(f"pip initialization failed: {e}") from e

    def install_dependencies(self, packages: list, dev_packages: list) -> None:
        """Install dependencies using pip.

        Note: This assumes pip is available and venv is activated.

        Args:
            packages (list): List of regular packages
            dev_packages (list): List of development packages
        """
        try:
            all_packages = packages + dev_packages
            if all_packages:
                subprocess.run(["pip", "install"] + all_packages, check=True)
        except subprocess.CalledProcessError as e:
            raise DependencyCreatorError(f"Failed to install dependencies: {e}") from e
        except FileNotFoundError as exc:
            raise DependencyCreatorNotFoundError(
                "pip not found. Please ensure Python and pip are installed."
            ) from exc



from datetime import datetime
from boilrpy.file_generators.base_generator import BaseGenerator


class LicenseGenerator(BaseGenerator):
    """
    Generator for license files.
    """

    def generate(self, *args, **kwargs) -> str:
        """
        Generate the content for the license file.

        :param license_name: Name of the license
        :param author: Name of the author
        :return: Content of the license file
        """
        license_name = args[0] if args else ""
        author = args[1] if args else ""
        license_strategy = self._get_license_strategy(license_name)
        return license_strategy.generate(author)

    def _get_license_strategy(self, license_name: str):
        """
        Get the appropriate license strategy based on the license name.

        :param license_name: Name of the license
        :return: LicenseStrategy object
        """
        strategies = {
            "mit": MITLicenseStrategy(),
            "apache": ApacheLicenseStrategy(),
            "gpl": GPLLicenseStrategy(),
            "bsd": BSDLicenseStrategy(),
        }
        return strategies.get(license_name.lower(), MITLicenseStrategy())


class LicenseStrategy:
    """
    Base class for license strategies.
    """

    def generate(self, *args) -> str:
        """
        Generate the license text.

        :param author: Name of the author
        :param kwargs: Additional keyword arguments
        :return: License text
        """
        author = args[0] if args else ""
        year = str(datetime.now().year)
        return self._get_template().format(year=year, author=author)

    def _get_template(self) -> str:
        """
        Get the license template.
        """
        raise NotImplementedError


class MITLicenseStrategy(LicenseStrategy):
    """
    Strategy for generating MIT license.
    """

    def _get_template(self) -> str:
        return """MIT License

Copyright (c) {year} {author}

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


class ApacheLicenseStrategy(LicenseStrategy):
    """
    Strategy for generating Apache license.
    """

    def _get_template(self) -> str:
        return """Apache License

Copyright {year} {author}

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


class GPLLicenseStrategy(LicenseStrategy):
    """
    Strategy for generating GPL license.
    """

    def _get_template(self) -> str:
        return """GPL License

Copyright {year} {author}

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <https://www.gnu.org/licenses/>.
"""


class BSDLicenseStrategy(LicenseStrategy):
    """
    Strategy for generating BSD license.
    """

    def _get_template(self) -> str:
        return """BSD License

Copyright (c) {year} {author}
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.
"""



from boilrpy.config import Config
from boilrpy.file_generator import FileGenerator
from boilrpy.file_writer import FileWriter


class FlaskAppCreator:
    """Class to create a new Flask app."""

    def __init__(self, config: Config):
        self.config = config
        self.file_generator = FileGenerator(config)
        self.file_writer = FileWriter(self.config.get_charset())

    def create_flask_project(self, project_info: dict) -> None:
        """Create a new Flask app.

        Args:
            project_info (dict): Dictionary containing project information
        """
        self._create_flask_folders()
        self._create_flask_app_file()
        self._create_flask_dot_env()
        self._create_flask_template_files(project_info)
        self._create_flask_static_files()

    def _create_flask_folders(self) -> None:
        self.file_writer.create_directory("templates")
        self.file_writer.create_directory("static/css")
        self.file_writer.create_directory("static/js")

    def _create_flask_app_file(self) -> None:
        content = self.file_generator.generate_flask_app_file()
        self.file_writer.write_file("app.py", content)

    def _create_flask_dot_env(self) -> None:
        content = self.file_generator.generate_dot_env_file()
        self.file_writer.write_file(".env", content)

    def _create_flask_template_files(self, project_info: dict) -> None:
        base_content = self.file_generator.generate_base_template(project_info)
        self.file_writer.write_file("templates/base.html", base_content)
        index_content = self.file_generator.generate_index_template(project_info)
        self.file_writer.write_file("templates/index.html", index_content)

    def _create_flask_static_files(self) -> None:
        css_content = self.file_generator.generate_style_file()
        self.file_writer.write_file("static/css/style.css", css_content)
        js_content = self.file_generator.generate_script_file()
        self.file_writer.write_file("static/js/script.js", js_content)



"""Factory for creating dependency manager creators."""

from typing import Type
from boilrpy.dependency_creators.base_dependency_creator import BaseDependencyCreator
from boilrpy.dependency_creators.poetry_creator import PoetryCreator
from boilrpy.dependency_creators.pip_creator import PipCreator
from boilrpy.dependency_creators.uv_creator import UvCreator
from boilrpy.dependency_creators.conda_creator import CondaCreator


class DependencyCreatorFactory:
    """Factory to create the appropriate dependency creator.

    This factory uses the Strategy pattern to instantiate the correct
    dependency creator based on the user's choice.

    Example:
        >>> factory = DependencyCreatorFactory()
        >>> creator = factory.create("poetry", config)
        >>> creator.create_dependency_file(project_info)
    """

    # Registry of available creators
    _creators: dict[str, Type[BaseDependencyCreator]] = {
        "poetry": PoetryCreator,
        "pip": PipCreator,
        "uv": UvCreator,
        "conda": CondaCreator,
    }

    @classmethod
    def create(cls, dep_manager: str, config) -> BaseDependencyCreator:
        """Create a dependency creator instance.

        Args:
            dep_manager: Name of the dependency manager (case-insensitive)
            config: Configuration object

        Returns:
            Instance of the appropriate dependency creator

        Raises:
            ValueError: If dependency manager is not supported

        Example:
            >>> creator = DependencyCreatorFactory.create("poetry", config)
            >>> isinstance(creator, PoetryCreator)
            True
        """
        dep_manager = dep_manager.lower()
        creator_class = cls._creators.get(dep_manager)

        if creator_class is None:
            supported = ", ".join(cls.get_supported_managers())
            raise ValueError(
                f"Unsupported dependency manager: '{dep_manager}'. "
                f"Supported managers: {supported}"
            )

        return creator_class(config)

    @classmethod
    def get_supported_managers(cls) -> list[str]:
        """Get list of supported dependency managers.

        Returns:
            Sorted list of supported dependency manager names

        Example:
            >>> managers = DependencyCreatorFactory.get_supported_managers()
            >>> "poetry" in managers
            True
        """
        return sorted(cls._creators.keys())

    @classmethod
    def register_creator(
        cls, name: str, creator_class: Type[BaseDependencyCreator]
    ) -> None:
        """Register a new dependency creator.

        This allows for dynamic registration of new creators without
        modifying the factory class.

        Args:
            name: Name of the dependency manager
            creator_class: Class of the creator (must inherit from BaseDependencyCreator)

        Raises:
            TypeError: If creator_class doesn't inherit from BaseDependencyCreator

        Example:
            >>> class PipenvCreator(BaseDependencyCreator):
            ...     pass
            >>> DependencyCreatorFactory.register_creator("pipenv", PipenvCreator)
        """
        if not issubclass(creator_class, BaseDependencyCreator):
            raise TypeError(
                f"{creator_class.__name__} must inherit from BaseDependencyCreator"
            )
        cls._creators[name.lower()] = creator_class

    @classmethod
    def is_supported(cls, dep_manager: str) -> bool:
        """Check if a dependency manager is supported.

        Args:
            dep_manager: Name of the dependency manager

        Returns:
            True if supported, False otherwise

        Example:
            >>> DependencyCreatorFactory.is_supported("poetry")
            True
            >>> DependencyCreatorFactory.is_supported("unknown")
            False
        """
        return dep_manager.lower() in cls._creators



from boilrpy.file_generators.base_generator import BaseGenerator


class MainFileGenerator(BaseGenerator):
    """
    Generator for main.py file.
    """

    def generate(self, *args, **kwargs) -> str:
        """
        Generate the content for main.py file.

        :return: Content of main.py file
        """
        return """def main():
    print("Welcome to your new project!")
    pass


if __name__ == '__main__':
    main()
"""



import os
import subprocess
from colorama import Fore, Style
from boilrpy.config import Config
from boilrpy.file_generator import FileGenerator
from boilrpy.file_writer import FileWriter
from boilrpy.utils.string_formatter import StringFormatter
from boilrpy.flask_app_creator import FlaskAppCreator
from boilrpy.dependency_creators import DependencyCreatorFactory


class ProjectCreator:
    """
    Class to create a new project.
    """

    def __init__(self, config: Config):
        self.project_name = None
        self.config = config
        self.charset = config.get_charset()
        self.file_generator = FileGenerator(config)
        self.file_writer = FileWriter(self.charset)

    def create_project(self, project_info: dict) -> None:
        """
        Create a new project.

        Args:
            project_info (dict): Dictionary containing project information

        Returns:
            None
        """
        self.project_name = StringFormatter.format_project_name(
            project_info["name"], self.config.use_camel_case
        )

        project_info["name"] = self.project_name
        project_info["version"] = project_info.get("version") or "0.1.0"

        if self._check_directory_exist(self.project_name):
            raise FileExistsError(f"Directory {self.project_name} already exists.")

        print(f"Creating project {self.project_name}...")
        project_path = self._create_project_directory()
        os.chdir(project_path)

        self._create_readme(project_info)

        self._create_license(project_info)

        self._create_gitignore()

        self._create_changelog(project_info["version"])

        self._create_dependency_files(project_info)

        self._create_dockerfile(project_info)

        self._create_test_folder(project_info["create_tests"])

        self._create_main_file(project_info["use_flask"])

        self._create_requirements_txt(project_info)

        self._create_linter_file(project_info["use_pylint"])

        self._create_flask_app(project_info)

        self._initialize_git_repository()

    def _create_project_directory(self) -> str:
        """
        Create a new project directory.

        Returns:
            str: The path to the new project directory.
        """
        project_path = os.path.join(os.getcwd(), self.project_name)
        self.file_writer.create_directory(project_path, exist_ok=False)
        return project_path

    def _create_readme(self, project_info: dict) -> None:
        """
        Create a new README.md file.

        Args:
            project_info (dict): Dictionary containing project information

        Returns:
            None
        """
        content = self.file_generator.generate_readme(project_info)
        self.file_writer.write_file("README.md", content)

    def _create_license(self, project_info: dict) -> None:
        if project_info["license"] == "None":
            return
        content = self.file_generator.generate_license(
            project_info["license"], project_info["author"]
        )
        self.file_writer.write_file("LICENSE", content)

    def _create_gitignore(self) -> None:
        content = self.file_generator.generate_gitignore()
        self.file_writer.write_file(".gitignore", content)

    def _create_changelog(self, version: str) -> None:
        content = self.file_generator.generate_changelog(version)
        self.file_writer.write_file("CHANGELOG.md", content)

    def _create_dependency_files(self, project_info: dict) -> None:
        """Create dependency configuration files based on selected manager.

        Args:
            project_info (dict): Dictionary containing project information
        """
        dep_manager = project_info.get("dependencies_manager", "pip")

        try:
            creator = DependencyCreatorFactory.create(dep_manager, self.config)
            creator.create_dependency_file(project_info)
        except ValueError as e:
            print(f"\n {e}")
            print("Falling back to pip...")
            creator = DependencyCreatorFactory.create("pip", self.config)
            creator.create_dependency_file(project_info)

    def _create_dockerfile(self, project_info: dict) -> None:
        if not project_info["use_docker"]:
            return
        content = self.file_generator.generate_dockerfile(
            self.project_name, project_info["use_flask"]
        )
        self.file_writer.write_file("Dockerfile", content)
        ignore_content = self.file_generator.generate_dockerignore()
        self.file_writer.write_file(".dockerignore", ignore_content)

    def _create_test_folder(self, create_tests: bool) -> None:
        if not create_tests:
            return
        self.file_writer.create_directory("tests")
        self.file_writer.write_file("tests/__init__.py", "")

    def _create_main_file(self, use_flask: bool) -> None:
        if use_flask:
            return
        content = self.file_generator.generate_main_file()
        self.file_writer.write_file("main.py", content)

    def _create_requirements_txt(self, project_info: dict) -> None:
        content = self.file_generator.generate_requirements_txt(project_info)
        self.file_writer.write_file("requirements.txt", content)

    def _create_linter_file(self, use_pylint: bool) -> None:
        if not use_pylint:
            return
        content = self.file_generator.generate_pylint()
        self.file_writer.write_file(".pylintrc", content)

    def _create_flask_app(self, project_info: dict) -> None:
        if not project_info["use_flask"]:
            return
        flask_creator = FlaskAppCreator(self.config)
        flask_creator.create_flask_project(project_info)

    def _initialize_git_repository(self) -> None:
        try:
            subprocess.run(["git", "--version"], check=True, capture_output=True)
            subprocess.run(["git", "init"], check=True)
        except FileNotFoundError:
            print(
                f"{Fore.YELLOW}Git not found on your system. "
                f"Skipping Git initialization.{Style.RESET_ALL}"
            )
            print(
                f"{Fore.YELLOW}To use Git with this project, please install Git "
                f"and run 'git init' manually in the project directory.{Style.RESET_ALL}"
            )
        except subprocess.CalledProcessError as e:
            print(f"{Fore.RED}Error initializing Git repository: {e}{Style.RESET_ALL}")
            print(
                f"{Fore.YELLOW}You may need to initialize the Git repository manually."
                f"{Style.RESET_ALL}"
            )

    def _check_directory_exist(self, directory: str) -> bool:
        return os.path.exists(directory) and os.path.isdir(directory)



from boilrpy.file_generators.base_generator import BaseGenerator


class ChangelogGenerator(BaseGenerator):
    """
    Generator for CHANGELOG.md file.
    """

    def generate(self, *args, **kwargs) -> str:
        """
        Generate the content for CHANGELOG.md file.

        :param version: Version number
        :return: Content of CHANGELOG.md file
        """
        version = args[0] if args else "0.0.1"
        template = """# Changelog

# ${version}

### Added
"""
        return self.render_template(template, version=version)



from abc import ABC, abstractmethod
from boilrpy.config import Config


class BaseDependencyCreator(ABC):
    """Base class for dependency managers creators."""

    def __init__(self, config: Config):
        self.config = config
        self.charset = self.config.get_charset()

    @abstractmethod
    def create_dependency_file(self, project_info: dict) -> None:
        """Create dependency configuration file.

        Args:
            project_info (dict): Dictionary containing project information
        """
        raise NotImplementedError("Subclasses must implement create_dependency_file")

    @abstractmethod
    def install_dependencies(self, packages: list, dev_packages: list) -> None:
        """Install dependencies.

        Args:
            packages (list): List of regular packages
            dev_packages (list): List of development packages
        """
        raise NotImplementedError("Subclasses must implement create_dependency_file")

    def _create_packages(self, project_info: dict) -> tuple[list, list]:
        """Create lists of packages and dev packages.

        Args:
            project_info (dict): Dictionary containing project information

        Returns:
            tuple: (packages, dev_packages)
        """
        dev_packages = ["pytest"] if project_info.get("create_tests") else []
        if project_info.get("use_pylint"):
            dev_packages.append("pylint")

        packages = []
        if project_info.get("use_flask"):
            packages.append("flask")
            packages.append("python-dotenv")

        # Add additional libraries
        additional_libs = project_info.get("libraries", [])
        if additional_libs:
            packages.extend(additional_libs)

        return packages, dev_packages

    def _write_requirements_files(
        self,
        packages: list,
        dev_packages: list,
        create_empty_if_no_packages: bool = True,
    ) -> None:
        """Write requirements.txt and requirements-dev.txt files.

        This is a common implementation used by pip-like dependency managers.

        Args:
            packages (list): List of regular packages
            dev_packages (list): List of development packages
            create_empty_if_no_packages (bool): Whether to create an empty
                requirements.txt if no packages are specified
        """
        # Create requirements.txt
        if packages:
            with open("requirements.txt", "w", encoding=self.charset) as f:
                for package in packages:
                    f.write(f"{package}\n")
        elif create_empty_if_no_packages:
            # Create empty requirements.txt with comment
            with open("requirements.txt", "w", encoding=self.charset) as f:
                f.write("# Add your dependencies here\n")

        # Create requirements-dev.txt
        if dev_packages:
            with open("requirements-dev.txt", "w", encoding=self.charset) as f:
                f.write("-r requirements.txt\n")
                for package in dev_packages:
                    f.write(f"{package}\n")


class DependencyCreatorNotFoundError(Exception):
    """Exception raised when dependency manager is not found."""


class DependencyCreatorError(Exception):
    """Exception raised when dependency manager operation fails."""



import subprocess
import toml
from boilrpy.dependency_creators.base_dependency_creator import (
    BaseDependencyCreator,
    DependencyCreatorNotFoundError,
    DependencyCreatorError,
)


class PoetryCreator(BaseDependencyCreator):
    """Class to create a new poetry project."""

    def create_dependency_file(self, project_info: dict) -> None:
        """Create a new poetry file.

        Args:
            project_info (dict): Dictionary containing project information
        """
        try:
            packages, dev_packages = self._create_packages(project_info)
            subprocess.run(["poetry", "init", "-n"], check=True)
            self._update_pyproject_toml(project_info)
            self.install_dependencies(packages, dev_packages)
        except FileNotFoundError as exc:
            raise DependencyCreatorNotFoundError(
                "Poetry not found. Please install Poetry and try again."
            ) from exc
        except subprocess.CalledProcessError as e:
            raise DependencyCreatorError(f"Poetry initialization failed: {e}") from e

    def install_dependencies(self, packages: list, dev_packages: list) -> None:
        """Install dependencies using Poetry.

        Args:
            packages (list): List of regular packages
            dev_packages (list): List of development packages
        """
        try:
            if dev_packages:
                subprocess.run(
                    ["poetry", "add", "--group", "dev"] + dev_packages, check=True
                )
            if packages:
                subprocess.run(["poetry", "add"] + packages, check=True)
        except subprocess.CalledProcessError as e:
            raise DependencyCreatorError(f"Failed to install dependencies: {e}") from e

    def _update_pyproject_toml(self, project_info):
        """Update pyproject.toml with project information."""
        pyproject_file = "pyproject.toml"

        with open(pyproject_file, "r", encoding=self.charset) as file:
            pyproject_data = toml.load(file)

        poetry_version = self._check_poetry_version()

        if poetry_version.startswith("1."):
            pyproject_data["tool"]["poetry"]["name"] = project_info["name"]
            pyproject_data["tool"]["poetry"]["version"] = project_info["version"]
            pyproject_data["tool"]["poetry"]["description"] = project_info[
                "description"
            ]
            pyproject_data["tool"]["poetry"]["authors"] = [project_info["author"]]
            pyproject_data["tool"]["poetry"]["license"] = project_info["license"]

        if poetry_version.startswith("2."):
            pyproject_data["project"]["name"] = project_info["name"]
            pyproject_data["project"]["version"] = project_info["version"]
            pyproject_data["project"]["description"] = project_info["description"]
            if project_info["author"]:
                pyproject_data["project"]["authors"] = [
                    {"name": project_info["author"]}
                ]
            pyproject_data["project"]["license"] = {"text": project_info["license"]}

        with open(pyproject_file, "w", encoding=self.charset) as file:
            toml.dump(pyproject_data, file)

    def _check_poetry_version(self):
        """Check Poetry version."""
        result = subprocess.run(
            ["poetry", "--version"],
            capture_output=True,
            text=True,
            check=True,
        )
        version_output = result.stdout.strip()
        return version_output.split()[-1]



from boilrpy.file_generators.base_generator import BaseGenerator


class RequirementsGenerator(BaseGenerator):
    """Generate the requirements.txt file."""

    def generate(self, *args, **kwargs) -> str:
        project_info = args[0] if args else {}
        package_map = {
            "pylint": "use_pylint",
            "flask": "use_flask",
            "python-dotenv": "use_flask",
            "pytest": "create_tests",
        }
        packages = [
            pkg for pkg, key in package_map.items() if project_info.get(key, False)
        ]
        return "\n".join(packages)



from .readme_generator import ReadmeGenerator
from .license_generator import LicenseGenerator
from .gitignore_generator import GitignoreGenerator
from .changelog_generator import ChangelogGenerator
from .main_file_generator import MainFileGenerator
from .dockerfile_generator import DockerfileGenerator

__all__ = [
    "ReadmeGenerator",
    "LicenseGenerator",
    "GitignoreGenerator",
    "ChangelogGenerator",
    "MainFileGenerator",
    "DockerfileGenerator",
]
